#ifndef _TELNET_LOOPER_H
#	define _TELNET_LOOPER_H

#include "TCPSockets.h"

/*
 * The command prefix, everything following this byte is a command as
 * defined below.
 */
#define TELNET_IAC		255

/*
 * NVT characters, as defined in RFC854.
 */
#define TELNET_NUL		0
#define TELNET_LF		10
#define TELNET_CR		13
#define TELNET_BEL		7
#define TELNET_BS		8
#define TELNET_HT		9
#define TELNET_VT		11
#define TELNET_FF		12

/*
 * Telnet Commands as defined in RFC854.
 */
#define TELNET_SE		240		// End of subnegotiation parameters.
#define TELNET_NOP		241		// No operation.
#define TELNET_DM		242		// Data portion of Sync operation.
#define TELNET_BRK		243		// NVT Break.
#define TELNET_IP		244		// Interrupt Proccess.
#define TELNET_AO		245		// Abort Output.
#define TELNET_AYT		246		// Are You There.
#define TELNET_EC		247		// Erase Character.
#define TELNET_EL		248		// Erase Line.
#define TELNET_GA		249		// The Go Ahead signal.
#define TELNET_SB		250		// Start subnegotiation parameters.
#define TELNET_WILL		251
#define TELNET_WONT		252
#define TELNET_DO		253
#define TELNET_DONT		254

/*
 * States for the various options.
 */
// Would have been nice for this to have been an enum, damn MWCC.
#define DO				0x01
#define DONT			0x02
#define WILL			0x04
#define WONT			0x08
#define REQUESTED		0x10

/*
 * Telnet options, for use with the WILL, WONT, DO, DONT commands.
 */
typedef enum {
	XMIT_BIN = 0,				// Request binary transmission - RFC856
	ECHO = 1,					// Request for echoing - RFC857
	NO_GA = 3,					// Request to turn off GA signal - RFC858
	STATUS = 5,					// Request status - RFC859
	TM = 6,						// Request timing mark - RFC860
	TERM = 24,					// Request terminal type - RFC884
	EOR = 25,					// Request end of record - RFC885
	NAWS = 31,					// Request window size negotiation.
	TTYSPEED = 32,				// Request terminal speed negotiation.
	RFLOW = 33,					// Request remote flow control.
	XLOC = 35,					// Request X display location negotiation.
	ENVIRON = 36,				// Request envioronment variables.
	AUTH = 37,					// Request authentication negotiation.
	EXOPL = 255					// Request extended options - RFC861
} option_t;

#define TELNET_MAXOPT	AUTH+1

class TelnetLooper: public TCPLooper {
private:
	bool parsing_commands;
	int options[TELNET_MAXOPT];
	int his_options[TELNET_MAXOPT];
	int want_state[TELNET_MAXOPT];

	inline void SendOption(option_t option, int state);
	void SetState(option_t option, int state);
	void SetHisState(option_t option, int state);
public:
	TelnetLooper();

	void InitConnection();
	void SendLocal(void *data, int32 nbytes);

	int GetState(option_t option);
	void SetWantState(option_t option, int state);
};

#endif
