#include "StatusView.h"

#include <interface/View.h>
#include <kernel/OS.h>

StatusView::StatusView(BRect frame, const char *name)
		   : BView(frame, name, B_FOLLOW_BOTTOM,
				   B_WILL_DRAW | B_FRAME_EVENTS)
{
	output_sem = create_sem(1, "output");
	stat_mess[0] = '\0';
}

void StatusView::AttachedToWindow()
{
	BView::AttachedToWindow();
	BFont font;
	GetFont(&font);
	font.GetHeight(&font_info);
	SetHighColor(0, 0, 0);
	SetLowColor(220, 220, 220);

	// Don't auto-draw.
	SetViewColor(B_TRANSPARENT_32_BIT);
}

void StatusView::Draw(BRect update_rect)
{
	BRect bounds = Bounds();
	FillRect(bounds, B_SOLID_LOW);

	// Draw the border.
	SetHighColor(150, 150, 150);
	StrokeLine(BPoint(bounds.left, bounds.top),
			   BPoint(bounds.right, bounds.top));
	StrokeLine(BPoint(bounds.right, bounds.top),
			   BPoint(bounds.right, bounds.bottom));
	SetHighColor(0, 0, 0);

	acquire_sem(output_sem);
	bounds.left += 2;
	bounds.top += font_info.ascent+2;
	DrawString(stat_mess, BPoint(bounds.left, bounds.top));
	release_sem(output_sem);
}

void StatusView::SetText(const char *string)
{
	acquire_sem(output_sem);
	strncpy(stat_mess, string, sizeof(stat_mess)-1);
	release_sem(output_sem);
	Window()->Lock();
	Invalidate(Bounds());
	Window()->Unlock();
}
