#include "TelnetApp.h"
#include "TelnetWindow.h"
#include "TelnetOpenWindow.h"
#include "TelnetMessages.h"
#include "TelnetErrors.h"

#include <interface/Alert.h>

bool TelnetApp::AddWindow(const char *host, const char *port)
{
	TelnetWindow *new_win;
	if (!(new_win = new TelnetWindow(BRect(32*(num_windows+1), 32*(num_windows+1),
											632+32*num_windows, 432+32*num_windows),
									  host, port)))
		return FALSE;

	num_windows++;
	return TRUE;
}

TelnetApp::TelnetApp() : BApplication("application/x-vnd.kublai-telnet")
{
	num_windows = 0;
}

void TelnetApp::ReadyToRun()
{
	// First things first, make sure the computer is in a state that can
	// run a program such as Telnet.
	if (!is_computer_on()) {
		syslog(LOG_CRIT, "Computer is off.");
		PostMessage(B_QUIT_REQUESTED);
	}
/*
 *	double fire_quotient = is_computer_on_fire();
 *	if (fire_quotient) {
 *		fprintf(stderr, "Error: Computer is on fire: %f.\n", fire_quotient);
 *		PostMessage(B_QUIT_REQUESTED);
 *	}
 */

	BPopUpMenu *main_menu = new BPopUpMenu("", FALSE, FALSE);
	BMenuItem *item = new BMenuItem("About Telnet", new BMessage(B_ABOUT_REQUESTED));
	item->SetTarget(this);
	main_menu->AddItem(item);

	main_menu->AddSeparatorItem();

	item = new BMenuItem("Open…", new BMessage(OPEN_MSG), 'O');
	item->SetTarget(this);
	main_menu->AddItem(item);
	item = new BMenuItem("Preference…", new BMessage(PREF_MSG));
	item->SetTarget(this);
	main_menu->AddItem(item);

	main_menu->AddSeparatorItem();

	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	item->SetTarget(this);
	main_menu->AddItem(item);

	// Now open up a dialog box asking for the host to connect to.
	if (!num_windows)
		PostMessage(OPEN_MSG);
}

void TelnetApp::AboutRequested()
{
	BAlert *about_box = new BAlert("", ABOUT_MESS, "OK", NULL, NULL, B_WIDTH_AS_USUAL,
								    B_INFO_ALERT);

	if (about_box)
		about_box->Go();
}

void TelnetApp::ArgvReceived(int32 argc, char **argv)
{
	BMessage msg(OPEN_CONNECT_MSG);

	if (argc > 3) {
		fprintf(stderr, "Usage: %s hostname [port].\n", argv[0]);
		PostMessage(B_QUIT_REQUESTED);
		return;
	} else if (argc == 3) {
		// We have a host and a port.
		msg.AddString("port", argv[2]);
	} else if (argc == 2) {
		// No port information.
		msg.AddString("port", "telnet");
	}
	msg.AddString("host", argv[1]);
	PostMessage(&msg);
}

void TelnetApp::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
	case OPEN_MSG:
		// Open up the open dialog box.
		TelnetOpenWindow *open_window = new TelnetOpenWindow(BPoint(50, 50));
		if (open_window)
			num_windows++;
		break;
	case PREF_MSG: {
		// Open up the prefs dialog box.
	}; break;
	case WIN_CLOSE_MSG: {
		if (!--num_windows)
			PostMessage(B_QUIT_REQUESTED);
	}; break;
	case OPEN_CONNECT_MSG:
		if (!AddWindow(msg->FindString("host"), msg->FindString("port"))) {
			syslog(LOG_ERR, "Couldn't create new window.");
			return;
		}
		if (!num_windows)
			PostMessage(B_QUIT_REQUESTED);
		break;
	case ARP_ABOUT_MSG: {
		BAlert *about_box  = new BAlert("", ABOUT_ARP_MESS, "OK", NULL, NULL,
										 B_WIDTH_AS_USUAL, B_INFO_ALERT);
		if (about_box)
			about_box->Go();
	}; break;
	default:
		BApplication::MessageReceived(msg);
	}
}