#include "TelnetOpenWindow.h"
#include "TelnetMessages.h"
#include "TelnetErrors.h"

#include <LayoutLib/HGroup.h>
#include <LayoutLib/MButton.h>
#include <LayoutLib/MTextControl.h>
#include <LayoutLib/MWindow.h>
#include <LayoutLib/Space.h>
#include <LayoutLib/VGroup.h>

#include <app/Message.h>

TelnetOpenWindow::TelnetOpenWindow(BPoint ul)
				 :MWindow(BRect(ul.x, ul.y, ul.x+250, ul.y), "Open", B_TITLED_WINDOW,
						  B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	host_control = new MTextControl(HOST_PROMPT, NULL);
	port_control = new MTextControl(PORT_PROMPT, "telnet");
	MButton *button = new MButton("Connect", new BMessage(OPEN_CONNECT_MSG), this,
								  minimax(-1, -1, -1, -1));
	button->MakeDefault(true);

	VGroup *view = new VGroup(
							  new HGroup(host_control, port_control, 0),
							  new HGroup(new Space(), button, 0),
							  0
							 );
	AddChild(view);

	host_control->MakeFocus(true);

	Show();
}

void TelnetOpenWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
	case OPEN_CONNECT_MSG: {
		BMessage app_msg(OPEN_CONNECT_MSG);

		const char *hostname = host_control->Text();
		if (!hostname || !*hostname) {
			BAlert *no_host = new BAlert("", NO_HOST_MESS, "OK", NULL, NULL,
										  B_WIDTH_AS_USUAL, B_STOP_ALERT);
			if (no_host)
				no_host->Go();

			Lock();
			host_control->MakeFocus();
			Unlock();
			break;
		}
		app_msg.AddString("host", hostname);

		const char *portname = port_control->Text();
		if (!portname || !*portname)
			app_msg.AddString("port", "telnet");
		else
			app_msg.AddString("port", portname);

		be_app->PostMessage(&app_msg);
		PostMessage(B_QUIT_REQUESTED);
	}; break;
	default:
		BWindow::MessageReceived(msg);
	}
}

bool TelnetOpenWindow::QuitRequested()
{
	if (BWindow::QuitRequested()) {
		be_app->PostMessage(WIN_CLOSE_MSG);
		return TRUE;
	}

	return FALSE;
}