#include "StatusView.h"
#include "TelnetWindow.h"
#include "TelnetMessages.h"
#include "TelnetErrors.h"
#include "TelnetLooper.h"

#include <TCPSockets.h>
#include <ArpCommon/ArpRemoteTerminal.h>
#include <ArpCommon/ArpTerminalInterface.h>
#include <ArpCommon/ArpEmulatorManager.h>
#include <interface/Menu.h>
#include <interface/MenuBar.h>
#include <interface/ScrollBar.h>
#include <interface/Window.h>

#include "TCPSockets.h"

#define STAT_FRAME_WIDTH 100

void TelnetWindow::AddMenuBar(BRect frame)
{
	menu_bar = new BMenuBar(frame, "window menu");

	// TODO: error check all these news.
	BMenu *file = new BMenu("File");
	BMenuItem *item = new BMenuItem("About…", new BMessage(B_ABOUT_REQUESTED));
	item->SetTarget(be_app);
	file->AddItem(item);
	item = new BMenuItem("About ArpTerminal…", new BMessage(ARP_ABOUT_MSG));
	item->SetTarget(be_app);
	file->AddItem(item);

	file->AddSeparatorItem();

	item = new BMenuItem("Open…", new BMessage(OPEN_MSG), 'O');
	item->SetTarget(be_app);
	file->AddItem(item);
	item = new BMenuItem("Close", new BMessage(B_QUIT_REQUESTED), 'W');
	item->SetTarget(this);
	file->AddItem(item);

	file->AddSeparatorItem();

	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	item->SetTarget(be_app);
	file->AddItem(item);

	edit_menu = new BMenu("Edit");

	BMenu *options = new BMenu("Options");
	item = new BMenuItem("Session…", new BMessage(WIN_PREFS_MSG));
	item->SetTarget(this);
	options->AddItem(item);
	options->AddSeparatorItem();
	item = new BMenuItem("Send Break", new BMessage(TCP_SEND_BRK));
	item->SetTarget(this);
	options->AddItem(item);
	item = new BMenuItem("Send Are You There", new BMessage(TCP_SEND_AYT));
	item->SetTarget(this);
	options->AddItem(item);

	emulation_menu = new BMenu("Emulation");

	// Now add the menus to the bar.
	menu_bar->AddItem(file);
	menu_bar->AddItem(edit_menu);
	menu_bar->AddItem(options);
	menu_bar->AddItem(emulation_menu);
	AddChild(menu_bar);
}

void TelnetWindow::ResizeToTerm(int32 rows, int32 cols)
{
	float width, height;

	if (!output_dev)
		return;

	output_dev->TermGetSize(rows == -1 ? &rows : NULL, cols == -1 ? &cols : NULL);
	output_dev->GetNeededSize(cols, rows, &width, &height);

	BRect frame = Frame();
	BRect term_frame = output_dev->Frame();
	width += floor(frame.Width() - term_frame.Width()) - 1;
	height += floor(frame.Height() - term_frame.Height()) - 1;
	ResizeTo(width, height);
	output_dev->Invalidate();
}

TelnetWindow::TelnetWindow(BRect frame, const char *name, const char *port)
			  : BWindow(frame, name, B_DOCUMENT_WINDOW, 0)
{
	fore_color.red = fore_color.green = fore_color.blue = fore_color.alpha = 0;
	back_color.red = back_color.green = back_color.blue = back_color.alpha = 255;

	// Make up a spiffy title.
	char title[255];
	if (!strcmp(port, "telnet") || !strcmp(port, "23"))
		strcpy(title, name);
	else
		sprintf(title, "%s [port %s]", name, port);
	SetTitle(title);

	// Convert to window coordinate system
	frame = Bounds();

	// Add the menu bar.
	BRect view_frame(-1, -1, 0, 0);
	AddMenuBar(view_frame);

	float width, height;
	menu_bar->GetPreferredSize(&width, &height);

	// Add the connection status view.
	view_frame = BRect(0, frame.bottom-B_H_SCROLL_BAR_HEIGHT+1, STAT_FRAME_WIDTH-1, frame.bottom);
	stat_view = new StatusView(view_frame, NULL);
	AddChild(stat_view);

	// Set the bounds for the view. Make sure you keep enough room for the
	// scroll bars and menu bars.
	view_frame = BRect(-1, height, frame.right-B_V_SCROLL_BAR_WIDTH,
					   frame.bottom-B_H_SCROLL_BAR_HEIGHT);
	//output_dev = new TelnetView(view_frame, name, port);
	output_dev = new ArpRemoteTerminal(view_frame, name);

	// Add the scroll bars
	view_frame = BRect(frame.right-B_V_SCROLL_BAR_WIDTH+1, height-1,
					   frame.right+1, frame.bottom-B_H_SCROLL_BAR_HEIGHT+1);
	vert_scroll = new BScrollBar(view_frame, "vert_scroll", output_dev, 0, 0, B_VERTICAL);
	AddChild(vert_scroll);

	view_frame = BRect(STAT_FRAME_WIDTH-1, frame.bottom-B_H_SCROLL_BAR_HEIGHT+1,
					   frame.right-B_V_SCROLL_BAR_WIDTH+1, frame.bottom+1);
	horiz_scroll = new BScrollBar(view_frame, "horiz_scroll", output_dev, 0, 0, B_HORIZONTAL);
	AddChild(horiz_scroll);

	// Must add the main view after attaching scroll bars to it.
	AddChild(output_dev);

	// Get ready to make a connection.
	// TODO: this should use getservbynam(), instead of hardwiring port 23.
	if (!strcmp(port, "telnet") || !strcmp(port, "23"))
		io_looper = new TelnetLooper;
	else
		io_looper = new TCPLooper;

	if (io_looper->Run() < 0) {
		// Argh! couldn't spawn another looper.
		BAlert *alert = new BAlert("", NO_SOCKET_THREAD_MESS, ERROR_BUTTON, NULL, NULL,
								   B_WIDTH_AS_USUAL, B_STOP_ALERT);
		alert->Go();
		
		io_looper = NULL;
		PostMessage(B_QUIT_REQUESTED);
		return;
	}

	// Init the socket. Create a BMessenger so we can get a reply.
	BMessenger messenger(io_looper);

	BMessage init_msg(TCP_INIT);
	init_msg.AddPointer(HANDLER_ARG, this);
	init_msg.AddString(HOST_ARG, name);
	init_msg.AddString(PORT_ARG, port);
	messenger.SendMessage(&init_msg, this);

	// Set up the ArpTerminal plugins search path.
	manager.AddEnvVar("ADDON_PATH", "ArpTerminal");
	manager.AddDirectory(B_BEOS_ADDONS_DIRECTORY, "ArpTerminal");
	manager.AddDirectory(B_COMMON_ADDONS_DIRECTORY, "ArpTerminal");
	manager.AddDirectory(B_USER_ADDONS_DIRECTORY, "ArpTerminal");

	// Now load them.
	manager.Start();
	manager.AddStaticEmulator(&ArpEmulator::AddonInterface);
	output_dev->UseEmulatorManager(&manager);

	// And set up the emulator menu.
	// TODO: put this into a dialog box.
	manager.CreateAddonMenu(emulation_menu);

	// Set up the Edit menu.
	BMenuItem *item = new BMenuItem("Copy", new BMessage(B_COPY), 'C');
	item->SetTarget(output_dev);
	edit_menu->AddItem(item);
	item = new BMenuItem("Paste", new BMessage(B_PASTE), 'V');
	item->SetTarget(output_dev);
	edit_menu->AddItem(item);
	edit_menu->AddSeparatorItem();
	item = new BMenuItem("Select All", new BMessage(B_SELECT_ALL), 'A');
	item->SetTarget(output_dev);
	edit_menu->AddItem(item);

	edit_menu->AddSeparatorItem();

	item = new BMenuItem("Preferences…", new BMessage(PREF_MSG));
	item->SetTarget(be_app);
	edit_menu->AddItem(item);

	// Set up the emulator we want.
	output_dev->SetEmulator("XTERM");
	output_dev->CheckEmulationMenu(emulation_menu);

	// Set up the terminal.
	output_dev->TermSetFixedSize(25, 80);
	output_dev->TermSetCursorPos(0, 0);
	// I'm not a fan of the default colors.
	output_dev->TermSetTextForeground(fore_color, 0);
	output_dev->TermSetTextBackground(back_color, 0);

	output_dev->SetRemote(this);

	// Input in the text view.
	Lock();
	output_dev->MakeFocus();
	Unlock();

	// Tell the world we're trying to connect
	stat_view->SetText("Connecting");

	ResizeToTerm();

	// Now show the window
	Show();
}

TelnetWindow::~TelnetWindow()
{
	fprintf(stderr, "DEBUG: Destructing TelnetWindow.\n");
}

bool TelnetWindow::QuitRequested()
{
	// Tell the app we died.
	be_app->PostMessage(WIN_CLOSE_MSG);
	return TRUE;
}

void TelnetWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
	case TCP_CONNECTED:
		stat_view->SetText("Connected");
		break;
	case TCP_NOCONN: {
		BAlert *alert = new BAlert("", READ_ERR_MESS, ERROR_BUTTON, NULL, NULL,
								    B_WIDTH_AS_USUAL, B_STOP_ALERT);
		alert->Go();
		PostMessage(B_QUIT_REQUESTED);
	}; break;
	case TCP_ERR: {
		int8 sock_err;
		char *err_mess;
		if (msg->FindInt8(ERROR_ARG, &sock_err) == B_OK) {
			switch (sock_err) {
			// TODO: Add the rest of the potential errors here, and do malloc() checking.
			case NO_CONNECT:
				err_mess = (char *)malloc(sizeof(NO_CONNECT_MESS)*sizeof(char));
				memcpy(err_mess, NO_CONNECT_MESS, sizeof(NO_CONNECT_MESS)*sizeof(char));
				break;
			case NO_SOCKET:
				err_mess = (char *)malloc(sizeof(NO_SOCKET_MESS)*sizeof(char));
				memcpy(err_mess, NO_SOCKET_MESS, sizeof(NO_SOCKET_MESS)*sizeof(char));
				break;
			case NO_HOST:
				err_mess = (char *)malloc(sizeof(NO_HOSTNAME_MESS)*sizeof(char));
				memcpy(err_mess, NO_HOSTNAME_MESS, sizeof(NO_HOSTNAME_MESS)*sizeof(char));
				break;
			}
		} else {
			err_mess = (char *)malloc(sizeof(SOCKET_INIT_ERR_MESS)*sizeof(char));
			memcpy(err_mess, SOCKET_INIT_ERR_MESS, sizeof(SOCKET_INIT_ERR_MESS)*sizeof(char));
		}
		stat_view->SetText("Closed");
		BAlert *alert = new BAlert("", err_mess, ERROR_BUTTON, NULL, NULL,
								    B_WIDTH_AS_USUAL, B_STOP_ALERT);
		alert->Go();

		free(err_mess);
		PostMessage(B_QUIT_REQUESTED);
	}; break;
	case TCP_RECVDATA: {
		int32 nbytes;
		char *data;
		if (msg->FindData(DATA_ARG, B_RAW_TYPE, &data, &nbytes) != B_OK)
			return;
		BMessage ttymsg(TERM_XFER_TEXT_MSG);
		ttymsg.AddData("text", B_ASCII_TYPE, data, nbytes);
		PostMessage(&ttymsg, output_dev);
	}; break;
	case TERM_XFER_TEXT_MSG: {
		int32 nbytes;
		char *data;
		if (msg->FindData("text", B_ASCII_TYPE, &data, &nbytes) != B_OK)
			return;
		SendRemote(data, nbytes);
	};
	default:
		BWindow::MessageReceived(msg);
	}
}

void TelnetWindow::SendRemote(const char *string, int32 nbytes)
{
	BMessage msg(TCP_SENDDATA);
	bool local_echo = TRUE;

	if (TelnetLooper *looper = dynamic_cast<TelnetLooper *>(io_looper))
		local_echo = looper->GetState(ECHO) & WILL;

	msg.AddData(DATA_ARG, B_RAW_TYPE, string, nbytes);
	io_looper->PostMessage(&msg);

	if (local_echo) {
		BMessage ttymsg(TERM_XFER_TEXT_MSG);
		ttymsg.AddData("text", B_ASCII_TYPE, string, nbytes);
		PostMessage(&ttymsg, output_dev);
	}
}